#include <algorithm>
#include <iostream>
#include <cassert>
#define int long long

const int maxn = 4000 + 10;

int pos[maxn], output[maxn];
std::string s[maxn];
int fact[maxn], invFact[maxn], base[maxn];
int n, m, mod;

int power(int x, int to)
{
    if (to == 0) return 1;
    if (to & 1) return (x * power(x, to-1)) % mod;
    int res = power(x, to/2);
    return (res * res) % mod;
}

int comb(int x, int y)
{
    if (y >= x+1) return 0;
    return (((fact[x] * invFact[y]) % mod) * invFact[x-y]) % mod;
}

int multBy[maxn];
int check0(int cnt0, int cnt1)
{
    int ans = 0;
    for (int i = 2 ; i <= cnt0 ; ++i)
    {
        for (int j = 0 ; j <= i-2 ; ++j)
        {
            ans += ((comb(cnt1, j) * comb(cnt0, i)) % mod) * multBy[i+j+1];
            ans %= mod;
        }
    }

    return ans;
}

int check1(int cnt0, int cnt1)
{
    int ans = 0;
    for (int i = 1 ; i <= cnt1 ; ++i)
    {
        for (int j = 0 ; j <= i-1 ; ++j)
        {
            // std::cout << "check1 for: " << comb(cnt0, j) << ' ' << comb(cnt1, i) << ' ' << multBy[i+j+1] << '\n';
            ans += ((comb(cnt0, j) * comb(cnt1, i)) % mod) * multBy[i+j+1];
            ans %= mod;
        }
    }

    return ans;
}

int solveFor(int ix)
{
    std::fill(multBy, multBy+n+1, 1);
    // std::cout << "solve for: " << ix << s[ix] << '\n';
    int l = 1, r = n, ans = 0;
    for (int i = 1 ; i <= m ; ++i)
    {
        // std::cout << "i: " << i << ' ' << l << ' ' << r << '\n';
        int cnt0 = 0;
        int cnt1 = 0;
        for (int j = l ; j <= r ; ++j)
        {
            if (j == ix) continue;
            cnt0 += (s[j][i] == '0');
            cnt1 += (s[j][i] == '1');
        }

        // std::cout << "cnt: " << cnt0 << ' ' << cnt1 << '\n';
        if (s[ix][i] == '0')
        {
            ans = (ans + check1(cnt0, cnt1)) % mod;
            int sum = 1;
            for (int i = 1 ; i <= n ; ++i)
            {
                if (i <= cnt1) multBy[i] *= sum;
                else multBy[i] *= base[cnt1];
                multBy[i] %= mod;
                sum += comb(cnt1, i);
            }
        }
        else
        {
            ans = (ans + check0(cnt0, cnt1)) % mod;
            int sum = 1;
            for (int i = 1 ; i <= n ; ++i)
            {
                sum += comb(cnt0, i);
                if (i <= cnt0) multBy[i] *= sum;
                else multBy[i] *= base[cnt0];
                multBy[i] %= mod;
            }
        }

        // std::cout << "done calcing: " << ans << "\n";


        while (s[l][i] != s[ix][i]) ++l;
        while (s[r][i] != s[ix][i]) --r;
    }
    return (base[n-1] + mod - ans) % mod;
}

bool cmp(int x, int y)
{
    return s[x] < s[y];
}

void solve()
{
    fact[0] = invFact[0] = base[0] = 1;
    for (int i = 1 ; i <= std::max(n, m) ; ++i)
    {
        pos[i] = i;
        fact[i] = (i * fact[i-1]) % mod;
        base[i] = (2 * base[i-1]) % mod;
        invFact[i] = power(fact[i], mod-2);
    }
    std::sort(pos+1, pos+1+n, cmp);
    std::sort(s+1, s+1+n);
    for (int i = 1 ; i <= n ; ++i)
    {
        output[pos[i]] = solveFor(i);
    }

    for (int i = 1 ; i <= n ; ++i)
    {
        std::cout << output[i] << '\n';
        assert(0 <= output[i] && output[i] <= mod-1);
    }
}

void read()
{
    std::cin >> n >> m >> mod;
    for (int i = 1 ; i <= n ; ++i)
    {
        std::cin >> s[i];
        s[i] = " " + s[i];
    }
}

void fastIO()
{
    std::ios_base :: sync_with_stdio(0);
    std::cin.tie(nullptr);
    std::cout.tie(nullptr);
}

signed main()
{
    fastIO();
    read();
    solve();

    return 0;
}

/**

3 2 11
10
11
10

4 4 29
1011
1001
0110
0101

*/
